@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif // defined(HAVE_CONFIG_H)
 
 #include <nghttp3/nghttp3.h>
 
@@ -41,4 +41,4 @@ struct Config {
 
 } // namespace nghttp3
 
-#endif // QPACK_H
+#endif // !defined(QPACK_H)
