@@ -32,12 +32,13 @@
 
 #include "nghttp3_conv.h"
 
-#define NGHTTP3_INITIAL_TABLE_LENBITS 4
+#define NGHTTP3_INITIAL_HASHBITS 4
 
-void nghttp3_map_init(nghttp3_map *map, const nghttp3_mem *mem) {
+void nghttp3_map_init(nghttp3_map *map, uint64_t seed, const nghttp3_mem *mem) {
   map->mem = mem;
   map->hashbits = 0;
   map->table = NULL;
+  map->seed = seed;
   map->size = 0;
 }
 
@@ -78,8 +79,14 @@ int nghttp3_map_each(const nghttp3_map *map, int (*func)(void *data, void *ptr),
   return 0;
 }
 
-static size_t hash(nghttp3_map_key_type key, size_t bits) {
-  return (size_t)((key * 11400714819323198485llu) >> (64 - bits));
+static size_t map_hash(const nghttp3_map *map, nghttp3_map_key_type key) {
+  /* hasher from
+     https://github.com/rust-lang/rustc-hash/blob/dc5c33f1283de2da64d8d7a06401d91aded03ad4/src/lib.rs
+     We do not perform finalization here because we use top bits
+     anyway. */
+  key += map->seed;
+  key *= 0xf1357aea2e62a9c5ull;
+  return (size_t)((key * 11400714819323198485llu) >> (64 - map->hashbits));
 }
 
 static void map_bucket_swap(nghttp3_map_bucket *a, nghttp3_map_bucket *b) {
@@ -110,28 +117,28 @@ void nghttp3_map_print_distance(const nghttp3_map *map) {
       continue;
     }
 
-    idx = hash(bkt->key, map->hashbits);
+    idx = map_hash(map, bkt->key);
     fprintf(stderr, "@%zu hash=%zu key=%" PRIu64 " base=%zu distance=%u\n", i,
-            hash(bkt->key, map->hashbits), bkt->key, idx, bkt->psl);
+            map_hash(map, bkt->key), bkt->key, idx, bkt->psl);
   }
 }
 #endif /* !defined(WIN32) */
 
-static int insert(nghttp3_map_bucket *table, size_t hashbits,
-                  nghttp3_map_key_type key, void *data) {
-  size_t idx = hash(key, hashbits);
+static int map_insert(nghttp3_map *map, nghttp3_map_key_type key, void *data) {
+  size_t idx = map_hash(map, key);
   nghttp3_map_bucket b = {
     .key = key,
     .data = data,
   };
   nghttp3_map_bucket *bkt;
-  size_t mask = (1u << hashbits) - 1;
+  size_t mask = (1u << map->hashbits) - 1;
 
   for (;;) {
-    bkt = &table[idx];
+    bkt = &map->table[idx];
 
     if (bkt->data == NULL) {
       *bkt = b;
+      ++map->size;
       return 0;
     }
 
@@ -152,15 +159,19 @@ static int insert(nghttp3_map_bucket *table, size_t hashbits,
 
 static int map_resize(nghttp3_map *map, size_t new_hashbits) {
   size_t i;
-  nghttp3_map_bucket *new_table;
   nghttp3_map_bucket *bkt;
   size_t tablelen;
   int rv;
+  nghttp3_map new_map = {
+    .table = nghttp3_mem_calloc(map->mem, 1u << new_hashbits,
+                                sizeof(nghttp3_map_bucket)),
+    .mem = map->mem,
+    .seed = map->seed,
+    .hashbits = new_hashbits,
+  };
   (void)rv;
 
-  new_table = nghttp3_mem_calloc(map->mem, 1u << new_hashbits,
-                                 sizeof(nghttp3_map_bucket));
-  if (new_table == NULL) {
+  if (new_map.table == NULL) {
     return NGHTTP3_ERR_NOMEM;
   }
 
@@ -173,15 +184,15 @@ static int map_resize(nghttp3_map *map, size_t new_hashbits) {
         continue;
       }
 
-      rv = insert(new_table, new_hashbits, bkt->key, bkt->data);
+      rv = map_insert(&new_map, bkt->key, bkt->data);
 
       assert(0 == rv);
     }
   }
 
   nghttp3_mem_free(map->mem, map->table);
+  map->table = new_map.table;
   map->hashbits = new_hashbits;
-  map->table = new_table;
 
   return 0;
 }
@@ -191,30 +202,28 @@ int nghttp3_map_insert(nghttp3_map *map, nghttp3_map_key_type key, void *data) {
 
   assert(data);
 
-  /* Load factor is 0.75 */
+  /* Load factor is 7/8 */
   /* Under the very initial condition, that is map->size == 0 and
-     map->hashbits == 0, 4 > 3 still holds nicely. */
-  if ((map->size + 1) * 4 > (1u << map->hashbits) * 3) {
+     map->hashbits == 0, 8 > 7 still holds nicely. */
+  if ((map->size + 1) * 8 > (1u << map->hashbits) * 7) {
     if (map->hashbits) {
       rv = map_resize(map, map->hashbits + 1);
       if (rv != 0) {
         return rv;
       }
     } else {
-      rv = map_resize(map, NGHTTP3_INITIAL_TABLE_LENBITS);
+      rv = map_resize(map, NGHTTP3_INITIAL_HASHBITS);
       if (rv != 0) {
         return rv;
       }
     }
   }
 
-  rv = insert(map->table, map->hashbits, key, data);
+  rv = map_insert(map, key, data);
   if (rv != 0) {
     return rv;
   }
 
-  ++map->size;
-
   return 0;
 }
 
@@ -228,7 +237,7 @@ void *nghttp3_map_find(const nghttp3_map *map, nghttp3_map_key_type key) {
     return NULL;
   }
 
-  idx = hash(key, map->hashbits);
+  idx = map_hash(map, key);
   mask = (1u << map->hashbits) - 1;
 
   for (;;) {
@@ -257,7 +266,7 @@ int nghttp3_map_remove(nghttp3_map *map, nghttp3_map_key_type key) {
     return NGHTTP3_ERR_INVALID_ARGUMENT;
   }
 
-  idx = hash(key, map->hashbits);
+  idx = map_hash(map, key);
   mask = (1u << map->hashbits) - 1;
 
   for (;;) {
