@@ -22,13 +22,11 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#ifndef NGTCP2_CONV_TEST_H
-#define NGTCP2_CONV_TEST_H
+#ifndef NGHTTP3_CONV_TEST_H
+#define NGHTTP3_CONV_TEST_H
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_nghttp3_pri_to_uint8(void);
-
-#endif /* NGTCP2_CONV_TEST_H */
+#endif /* !defined(NGHTTP3_CONV_TEST_H) */
